#include <avr/io.h>
#define F_CPU   8e6                                   // CPU Speed 8MHz
#include <util/delay.h>

void beep(void)
{
    DDRC = DDRC |=  (1<<PC3);

    uint16_t i;
    for (i=0; i<500; i++)
    {
        PORTC = PORTC |=  (1<<PC3);
        _delay_ms(1);
        PORTC = PORTC &= ~(1<<PC3);
        _delay_ms(1);
    }
    DDRC = DDRC &= ~(1<<PC3);
}


uint16_t readPin(uint8_t mux)
{
    uint8_t i;
    uint16_t result = 0;                                    //Initialisieren wichtig, da lokale Variablen
                                                            //nicht automatisch initialisiert werden und
                                                            //zufllige Werte haben. Sonst kann Quatsch rauskommen
    ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);           // Frequenzvorteiler 
                                                            // setzen auf 8 (1) und ADC aktivieren (1)

    ADMUX = mux;                                            // Kanal waehlen
//    ADMUX |= (1<<REFS1) | (1<<REFS0);                     // interne Referenzspannung nutzen 
    ADMUX |= (0<<REFS1) | (1<<REFS0);                       // Ub Referenzspannung nutzen 

                                                            // nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
                                                            // also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen"
    ADCSRA |= (1<<ADSC);                                    // eine ADC-Wandlung 
    while ( ADCSRA & (1<<ADSC) ) {
     ;                                                      // auf Abschluss der Konvertierung warten 
    }

                                                            // Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen
    for(i=0;i<4;i++)
    {
    ADCSRA |= (1<<ADSC);                                    // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;                                                     // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		                                    // Wandlungsergebnisse aufaddieren
    }
    ADCSRA &= ~(1<<ADEN);                                   // ADC deaktivieren (2)

    result /= 4;                                            // Summe durch vier teilen = arithm. Mittelwert

    return result;
}



void uint2BCD(uint16_t zahl, char *sT)
{
    uint16_t zahl1 = zahl;
    uint16_t zahl2 = 0;

    uint8_t zehntausend = 0;
    uint8_t tausend = 0;
    uint8_t hundert = 0;
    uint8_t zehn = 0;
    uint8_t ein = 0;

    zahl2 = zahl1 % 10000;
    zehntausend = (zahl1 - zahl2) / 10000;
    zahl1 = zahl2;

    zahl2 = zahl1 % 1000;
    tausend = (zahl1 - zahl2) / 1000;
    zahl1 = zahl2;

    zahl2 = zahl1 % 100;
    hundert = (zahl1 - zahl2) / 100;
    zahl1 = zahl2;

    zahl2 = zahl1 % 10;
    zehn = (zahl1 - zahl2) / 10;
    zahl1 = zahl2;

    zahl2 = zahl1 % 1;
    ein = (zahl1 - zahl2) / 1;

    sT[0] = (zehntausend + 0x30);
    sT[1] = (tausend + 0x30);
    sT[2] = (hundert + 0x30);
    sT[3] = (zehn + 0x30);
    sT[4] = (ein + 0x30);
    sT[5] = 0;
return;
}



uint16_t tempAusgeben()
{
    char sTempASC[6];
    uint16_t iTemp = readPin(PA2);
//  uint16_t temp = 12345;
    uint2BCD(iTemp, sTempASC);
    lcd_gotoxy (0,2);
    lcd_puts(sTempASC);
    return iTemp;
}



void keysAusgeben()
{
    char * sKeysASC;
    uint16_t iKeys = readPin(PA4);

    sKeysASC=" - ";

    if (686<iKeys && iKeys<706)
        sKeysASC="1  ";
    if (500<iKeys && iKeys<520)
        sKeysASC=" 2 ";
    if (312<iKeys && iKeys<332)
        sKeysASC="  3";
    if (764<iKeys && iKeys<784)
        sKeysASC="12 ";
    if (596<iKeys && iKeys<616)
        sKeysASC=" 23";
    if (789<iKeys && iKeys<809)
        sKeysASC="123";
    if (726<iKeys && iKeys<746)
        sKeysASC="1 3";

    lcd_gotoxy (6,2);
    lcd_puts(sKeysASC);
}


void humAusgeben()
{
    char sHumASC[6];
    uint16_t iTemp = readPin(PA0);
    uint2BCD(iTemp, sHumASC);
    lcd_gotoxy (11,2);
    lcd_puts(sHumASC);
}




void LEDsSetzen(uint16_t iTemp)
{
    if (iTemp < 540)
    {
        LEDrt0();
        LEDge0();
        LEDgn1();
    };
    if ((iTemp > 540) && (iTemp < 570))
    {
        LEDrt0();
        LEDge1();
        LEDgn0();
    };
    if (iTemp > 571) 
    {
        LEDrt1();
        LEDge0();
        LEDgn0();
    };
    return;
}



void LEDgn1(void)
{
    DDRC  = DDRC   |= (1<<PC5);
    PORTC = PORTC  |= (1<<PC5);
}
void LEDgn0(void)
{
    DDRC  = DDRC  &= ~(1<<PC5);
    PORTC = PORTC &= ~(1<<PC5);
}
void LEDrt1(void)
{
    DDRC  = DDRC   |= (1<<PC6);
    PORTC = PORTC  |= (1<<PC6);
}
void LEDrt0(void)
{
    DDRC  = DDRC  &= ~(1<<PC6);
    PORTC = PORTC &= ~(1<<PC6);
}
void LEDge1(void)
{
    DDRC  = DDRC   |= (1<<PC7);
    PORTC = PORTC  |= (1<<PC7);
}
void LEDge0(void)
{
    DDRC  = DDRC  &= ~(1<<PC7);
    PORTC = PORTC &= ~(1<<PC7);
}


void wait_ms(int ms)
{
    int i;
    for (i=0;i<ms;i++)
    {
        _delay_ms(1);
    }
}
